# -*- coding: utf-8 -*-
"""a custom canvas - draws given data as a barchart into a userarea"""


import c4d
from c4d import documents
from c4d import gui
from c4d import plugins

from smartpolychart import ids


class Canvas(gui.GeUserArea):

    # the object list supplied by the worker
    data = []

    # step keeping and length for supersimple animation
    step = 0
    steps = 10

    # width of an item and the text widthin
    blockWidth = 0
    fontHeight = 0

    # the index of a clicked item (<0 for none)
    selected = -1

    # called on Redraw()
    def DrawMsg(self, x1, y1, x2, y2, msg):

        # set font height and item with
        self.fontHeight = self.DrawGetFontHeight()
        self.blockWidth = self.fontHeight + 2

        # set offscreen to define the whole canvas as clipping region
        #(normally this would suffice for this type of graphic since
        #we will end up drawing on the whole 'screen' anyway - but the
        #text will get a clipping region so it doesn't overlap the outlines)
        self.OffScreenOn()

        # draw a brackground
        self.DrawSetPen(c4d.COLOR_BG_DARK2)
        self.DrawRectangle(0, 0, x2, y2)

        # display a notification of there were no objects found and return
        if len(self.data) < 1:
            txt = c4d.plugins.GeLoadString(ids.STR_NOOBJECTS_FOUND)
            self.DrawSetTextCol(c4d.COLOR_TEXT, c4d.COLOR_TRANS)
            self.DrawText(txt,
                          x2 / 2 - self.DrawGetTextWidth(txt) / 2,
                          y2 / 2 - self.fontHeight / 2)

            return

        # rotate the text-alignment (DONT! forget to reset this)
        self.DrawSetTextRotation(-90)

        # offsets for text position and a small border
        txtOffsetY = 5
        fillBorder = 1

        # the max polycount
        maxValue = self.data[0][0]

        # iterate all objects...
        for i, dat in enumerate(self.data):
            # (re) set the text color to black
            self.DrawSetTextCol(c4d.COLOR_TEXT, c4d.COLOR_TRANS)

            # scale the actual poly-value to the animation progress
            value = float(dat[0]) / self.steps * self.step

            # the columns offset
            tmpOffsetX = i * self.blockWidth

            # stop drawing if the element is outside of the visible area
            if tmpOffsetX > x2: break
            
            # set the clipping region to the column we work on
            self.SetClippingRegion(tmpOffsetX, 0,
                                   tmpOffsetX + self.blockWidth,
                                   y2)

            # draw an orange border for this column
            self.DrawBorder(c4d.BORDER_ACTIVE_4,
                            tmpOffsetX,
                            0,
                            tmpOffsetX + self.blockWidth,
                            y2 - 1)

            # determine offsets and height for an inner, colored bar  
            innerHeight = y2 - fillBorder * 2 - 1
            ty = int(innerHeight - (float(innerHeight) / maxValue * value))
            
            # tint the bar with a color range from yellow to red
            channelValue = 1.0 / maxValue * value
            self.DrawSetPen(c4d.Vector(channelValue, 255 - channelValue, 0))
            
            # draw the value bar
            self.DrawRectangle(tmpOffsetX + fillBorder, ty + fillBorder,
                               tmpOffsetX + self.blockWidth - fillBorder,
                               y2 - fillBorder - 1)

            # clip the text a pixel earlier so it doesn't overlap the outline 
            self.SetClippingRegion(tmpOffsetX, 1,
                                   tmpOffsetX + self.blockWidth,
                                   y2)
            
            # color the object if it was just clicked
            if i == self.selected:
                self.DrawSetTextCol(c4d.COLOR_TEXT_SELECTED_DARK,
                                    c4d.COLOR_TRANS)
                self.selected = -1

            # draw poly cound and object name 
            self.DrawText(c4d.plugins.GeLoadString(ids.STR_COLUMN_TITLE) % (dat[0], dat[1]),
                           tmpOffsetX,
                           y2 - txtOffsetY)

        # ...remember?
        self.DrawSetTextRotation(0)

    def InputEvent(self, msg):

        # get the input device id
        dev = msg.GetLong(c4d.BFM_INPUT_DEVICE)

        # we dont hadle keyboard-...
        if dev == c4d.BFM_INPUT_KEYBOARD:
            return False
        
        # ...but mouse events
        if dev == c4d.BFM_INPUT_MOUSE:
            return self.HandleMouseEvents(msg)

        return False

    def HandleMouseEvents(self, msg):

        # get the input channel - we need this to...
        chn = msg.GetLong(c4d.BFM_INPUT_CHANNEL)

        # ...check for the left mouse button
        if chn == c4d.BFM_INPUT_MOUSELEFT:
            # get the mouse position and transform it into the canvas space
            mx = msg.GetLong(c4d.BFM_INPUT_X)
            my = msg.GetLong(c4d.BFM_INPUT_Y)    
            mx = mx - self.Local2Global()['x']
            my = my - self.Local2Global()['y']
            
            if mx < len(self.data) * self.blockWidth:
                # get the corresponding element to the clicked column
                index = int(mx / self.blockWidth)
                dat = self.data[index]
    
                # the object was removed since the last update - > return
                if not dat[2].IsAlive(): return False
        
                # select the clicked object
                doc = documents.GetActiveDocument()
                if doc is None: return False
    
                doc.SetActiveObject(dat[2])
                c4d.EventAdd()
                
                # mark this object as selected...
                self.selected = index
                
                # ...and force a redraw to highlight it
                self.Redraw()
                
                # queue another redraw via the timer to unhighlight it 
                self.SetTimer(100)
                
                return True

        return False

    # starts the animation
    def draw(self, aData):

        self.data = aData
        self.step = 0

        self.SetTimer(10)
        

    # calls a redraw, increments the step-value while animating
    #and stops the animation once steps is reached
    def Timer(self, msg):
        
        if self.step < self.steps: self.step += 1

        self.Redraw()

        if self.step >= self.steps: self.SetTimer(0)
